@@ -1,28 +1,52 @@
 import json
 import sys
-from pprint import pformat
 
 import requests
+from requests.adapters import HTTPAdapter
 from requests.packages import urllib3
 
 from httpie import sessions
 from httpie import __version__
 from httpie.compat import str
+from httpie.input import SSL_VERSION_ARG_MAPPING
 from httpie.plugins import plugin_manager
+from httpie.utils import repr_dict_nice
+
+try:
+    # https://urllib3.readthedocs.io/en/latest/security.html
+    urllib3.disable_warnings()
+except AttributeError:
+    # In some rare cases, the user may have an old version of the requests
+    # or urllib3, and there is no method called "disable_warnings." In these
+    # cases, we don't need to call the method.
+    # They may get some noisy output but execution shouldn't die. Move on.
+    pass
 
 
-# https://urllib3.readthedocs.org/en/latest/security.html
-urllib3.disable_warnings()
-
-
-FORM = 'application/x-www-form-urlencoded; charset=utf-8'
-JSON = 'application/json'
+FORM_CONTENT_TYPE = 'application/x-www-form-urlencoded; charset=utf-8'
+JSON_CONTENT_TYPE = 'application/json'
+JSON_ACCEPT = '{0}, */*'.format(JSON_CONTENT_TYPE)
 DEFAULT_UA = 'HTTPie/%s' % __version__
 
 
-def get_requests_session():
+class HTTPieHTTPAdapter(HTTPAdapter):
+
+    def __init__(self, ssl_version=None, **kwargs):
+        self._ssl_version = ssl_version
+        super(HTTPieHTTPAdapter, self).__init__(**kwargs)
+
+    def init_poolmanager(self, *args, **kwargs):
+        kwargs['ssl_version'] = self._ssl_version
+        super(HTTPieHTTPAdapter, self).init_poolmanager(*args, **kwargs)
+
+
+def get_requests_session(ssl_version):
     requests_session = requests.Session()
-    for cls in plugin_manager.get_trasnsport_plugins():
+    requests_session.mount(
+        'https://',
+        HTTPieHTTPAdapter(ssl_version=ssl_version)
+    )
+    for cls in plugin_manager.get_transport_plugins():
         transport_plugin = cls()
         requests_session.mount(prefix=transport_plugin.prefix,
                                adapter=transport_plugin.get_adapter())
@@ -32,7 +56,12 @@ def get_requests_session():
 def get_response(args, config_dir):
     """Send the request and return a `request.Response`."""
 
-    requests_session = get_requests_session()
+    ssl_version = None
+    if args.ssl_version:
+        ssl_version = SSL_VERSION_ARG_MAPPING[args.ssl_version]
+
+    requests_session = get_requests_session(ssl_version)
+    requests_session.max_redirects = args.max_redirects
 
     if not args.session and not args.session_read_only:
         kwargs = get_requests_kwargs(args)
@@ -52,17 +81,27 @@ def get_response(args, config_dir):
 
 
 def dump_request(kwargs):
-    sys.stderr.write('\n>>> requests.request(%s)\n\n'
-                     % pformat(kwargs))
+    sys.stderr.write('\n>>> requests.request(**%s)\n\n'
+                     % repr_dict_nice(kwargs))
 
 
-def encode_headers(headers):
-    # This allows for unicode headers which is non-standard but practical.
-    # See: https://github.com/jakubroztocil/httpie/issues/212
-    return dict(
-        (name, value.encode('utf8') if isinstance(value, str) else value)
-        for name, value in headers.items()
-    )
+def finalize_headers(headers):
+    final_headers = {}
+    for name, value in headers.items():
+        if value is not None:
+
+            # >leading or trailing LWS MAY be removed without
+            # >changing the semantics of the field value"
+            # -https://www.w3.org/Protocols/rfc2616/rfc2616-sec4.html
+            # Also, requests raises `InvalidHeader` for leading spaces.
+            value = value.strip()
+
+            if isinstance(value, str):
+                # See: https://github.com/jkbrzt/httpie/issues/212
+                value = value.encode('utf8')
+
+        final_headers[name] = value
+    return final_headers
 
 
 def get_default_headers(args):
@@ -71,16 +110,15 @@ def get_default_headers(args):
     }
 
     auto_json = args.data and not args.form
-    # FIXME: Accept is set to JSON with `http url @./file.txt`.
     if args.json or auto_json:
-        default_headers['Accept'] = 'application/json'
+        default_headers['Accept'] = JSON_ACCEPT
         if args.json or (auto_json and args.data):
-            default_headers['Content-Type'] = JSON
+            default_headers['Content-Type'] = JSON_CONTENT_TYPE
 
     elif args.form and not args.files:
         # If sending files, `requests` will set
         # the `Content-Type` for us.
-        default_headers['Content-Type'] = FORM
+        default_headers['Content-Type'] = FORM_CONTENT_TYPE
     return default_headers
 
 
@@ -92,7 +130,7 @@ def get_requests_kwargs(args, base_headers=None):
     # Serialize JSON data, if needed.
     data = args.data
     auto_json = data and not args.form
-    if args.json or auto_json and isinstance(data, dict):
+    if (args.json or auto_json) and isinstance(data, dict):
         if data:
             data = json.dumps(data)
         else:
@@ -105,12 +143,7 @@ def get_requests_kwargs(args, base_headers=None):
     if base_headers:
         headers.update(base_headers)
     headers.update(args.headers)
-    headers = encode_headers(headers)
-
-    credentials = None
-    if args.auth:
-        auth_plugin = plugin_manager.get_auth_plugin(args.auth_type)()
-        credentials = auth_plugin.get_auth(args.auth.key, args.auth.value)
+    headers = finalize_headers(headers)
 
     cert = None
     if args.cert:
@@ -130,7 +163,7 @@ def get_requests_kwargs(args, base_headers=None):
         }.get(args.verify, args.verify),
         'cert': cert,
         'timeout': args.timeout,
-        'auth': credentials,
+        'auth': args.auth,
         'proxies': dict((p.key, p.value) for p in args.proxy),
         'files': args.files,
         'allow_redirects': args.follow,
