@@ -7,6 +7,7 @@ from __future__ import division
 import os
 import re
 import sys
+import errno
 import mimetypes
 import threading
 from time import sleep, time
@@ -77,15 +78,15 @@ def parse_content_range(content_range, resumed_from):
     # last-byte-pos value, is invalid. The recipient of an invalid
     # byte-content-range- spec MUST ignore it and any content
     # transferred along with it."
-    if (first_byte_pos >= last_byte_pos
-            or (instance_length is not None
-                and instance_length <= last_byte_pos)):
+    if (first_byte_pos >= last_byte_pos or
+            (instance_length is not None and
+             instance_length <= last_byte_pos)):
         raise ContentRangeError(
             'Invalid Content-Range returned: %r' % content_range)
 
-    if (first_byte_pos != resumed_from
-        or (instance_length is not None
-            and last_byte_pos + 1 != instance_length)):
+    if (first_byte_pos != resumed_from or
+            (instance_length is not None and
+             last_byte_pos + 1 != instance_length)):
         # Not what we asked for.
         raise ContentRangeError(
             'Unexpected Content-Range returned (%r)'
@@ -104,7 +105,7 @@ def filename_from_content_disposition(content_disposition):
     :return: the filename if present and valid, otherwise `None`
 
     """
-    # attachment; filename=jakubroztocil-httpie-0.4.1-20-g40bd8f6.tar.gz
+    # attachment; filename=jkbrzt-httpie-0.4.1-20-g40bd8f6.tar.gz
 
     msg = Message('Content-Disposition: %s' % content_disposition)
     filename = msg.get_filename()
@@ -135,16 +136,51 @@ def filename_from_url(url, content_type):
     return fn
 
 
+def trim_filename(filename, max_len):
+    if len(filename) > max_len:
+        trim_by = len(filename) - max_len
+        name, ext = os.path.splitext(filename)
+        if trim_by >= len(name):
+            filename = filename[:-trim_by]
+        else:
+            filename = name[:-trim_by] + ext
+    return filename
+
+
+def get_filename_max_length(directory):
+    max_len = 255
+    try:
+        pathconf = os.pathconf
+    except AttributeError:
+        pass  # non-posix
+    else:
+        try:
+            max_len = pathconf(directory, 'PC_NAME_MAX')
+        except OSError as e:
+            if e.errno != errno.EINVAL:
+                raise
+    return max_len
+
+
+def trim_filename_if_needed(filename, directory='.', extra=0):
+    max_len = get_filename_max_length(directory) - extra
+    if len(filename) > max_len:
+        filename = trim_filename(filename, max_len)
+    return filename
+
+
 def get_unique_filename(filename, exists=os.path.exists):
     attempt = 0
     while True:
         suffix = '-' + str(attempt) if attempt > 0 else ''
-        if not exists(filename + suffix):
-            return filename + suffix
+        try_filename = trim_filename_if_needed(filename, extra=len(suffix))
+        try_filename += suffix
+        if not exists(try_filename):
+            return try_filename
         attempt += 1
 
 
-class Download(object):
+class Downloader(object):
 
     def __init__(self, output_file=None,
                  resume=False, progress_file=sys.stderr):
@@ -178,8 +214,8 @@ class Download(object):
         :type request_headers: dict
 
         """
-        # Disable content encoding so that we can resume, etc.
-        request_headers['Accept-Encoding'] = None
+        # Ask the server not to encode the content so that we can resume, etc.
+        request_headers['Accept-Encoding'] = 'identity'
         if self._resume:
             bytes_have = os.path.getsize(self._output_file.name)
             if bytes_have:
@@ -201,6 +237,8 @@ class Download(object):
         """
         assert not self.status.time_started
 
+        # FIXME: some servers still might sent Content-Encoding: gzip
+        # <https://github.com/jkbrzt/httpie/issues/423>
         try:
             total_size = int(response.headers['Content-Length'])
         except (KeyError, ValueError, TypeError):
@@ -270,9 +308,9 @@ class Download(object):
     @property
     def interrupted(self):
         return (
-            self.finished
-            and self.status.total_size
-            and self.status.total_size != self.status.downloaded
+            self.finished and
+            self.status.total_size and
+            self.status.total_size != self.status.downloaded
         )
 
     def chunk_downloaded(self, chunk):
@@ -299,8 +337,7 @@ class Status(object):
 
     def started(self, resumed_from=0, total_size=None):
         assert self.time_started is None
-        if total_size is not None:
-            self.total_size = total_size
+        self.total_size = total_size
         self.downloaded = self.resumed_from = resumed_from
         self.time_started = time()
 
@@ -362,8 +399,8 @@ class ProgressReporterThread(threading.Thread):
         if now - self._prev_time >= self._update_interval:
             downloaded = self.status.downloaded
             try:
-                speed = ((downloaded - self._prev_bytes)
-                         / (now - self._prev_time))
+                speed = ((downloaded - self._prev_bytes) /
+                         (now - self._prev_time))
             except ZeroDivisionError:
                 speed = 0
 
@@ -397,11 +434,11 @@ class ProgressReporterThread(threading.Thread):
             self._prev_bytes = downloaded
 
         self.output.write(
-            CLEAR_LINE
-            + ' '
-            + SPINNER[self._spinner_pos]
-            + ' '
-            + self._status_line
+            CLEAR_LINE +
+            ' ' +
+            SPINNER[self._spinner_pos] +
+            ' ' +
+            self._status_line
         )
         self.output.flush()
 
@@ -410,8 +447,8 @@ class ProgressReporterThread(threading.Thread):
                              else 0)
 
     def sum_up(self):
-        actually_downloaded = (self.status.downloaded
-                               - self.status.resumed_from)
+        actually_downloaded = (
+            self.status.downloaded - self.status.resumed_from)
         time_taken = self.status.time_finished - self.status.time_started
 
         self.output.write(CLEAR_LINE)
@@ -426,8 +463,8 @@ class ProgressReporterThread(threading.Thread):
 
         self.output.write(SUMMARY.format(
             downloaded=humanize_bytes(actually_downloaded),
-            total=(self.status.total_size
-                   and humanize_bytes(self.status.total_size)),
+            total=(self.status.total_size and
+                   humanize_bytes(self.status.total_size)),
             speed=humanize_bytes(speed),
             time=time_taken,
         ))
